// WatchDog.cpp
// ADS 720020-1195x
//
// Code for the WatchDog class to use the WatchDog Timer on the EP9315 CPU
//
// jn 15 Aug 2006

#include "watchdog.h"

#define PAGE_SIZE 				0x1000 	// 1 page:4KB
#define PAGE_ALIGN(Vaddr) 		((PVOID)((ULONG)(Vaddr) & ~(PAGE_SIZE - 1)))
#define ROUND_TO_PAGES(Size)  	(((ULONG)(Size) + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1))

extern "C"
{
   __declspec(dllimport) BOOL   VirtualCopy(LPVOID lpvDest, LPVOID lpvSrc, DWORD cbSize, DWORD fdwProtect);
}

// forward declarations
PVOID MapPhysicalAddr(PVOID dwPhysicalAddress, DWORD dwSize);

WatchDog::WatchDog(ULONG interval)
{
	DEBUGMSG(TRUE, (_T("+WatchDog::WatchDog(0x%08x)\r\n"), interval));	

	m_wd = (WD *)MapPhysicalAddr((void *)EP9315_OST_PHYSICAL_BASE, 0x1000);
	m_enabled = FALSE;

	DEBUGMSG(TRUE, (_T("-WatchDog::WatchDog()\r\n")));
}

WatchDog::~WatchDog()
{
	DEBUGMSG(TRUE, (_T("+WatchDog::~WatchDog()\r\n")));

	// Disable Watchdog Timer
	//---------------------------
	WRITE_REGISTER_ULONG(&m_wd->REG, WD_DISABLE);

	if(m_wd)
		VirtualFree(m_wd, 0, MEM_RELEASE);

	DEBUGMSG(TRUE, (_T("-WatchDog::~WatchDog()\r\n")));
}

void WatchDog::Enable()
{
	DEBUGMSG(TRUE, (_T("+WatchDog::Enable()\r\n")));	

	// Enable Watchdog Timer
	//-----------------------
	DEBUGMSG(TRUE, (_T("\tEnabling the watchdog\r\n")));
	WRITE_REGISTER_ULONG(&m_wd->REG, WD_ENABLE);
	m_enabled = TRUE;

	DEBUGMSG(TRUE, (_T("-WatchDog::Enable()\r\n")));
}

void WatchDog::Disable()
{
	DEBUGMSG(TRUE, (_T("+WatchDog::Disable()\r\n")));	

	// Disable Watchdog Timer
	//-----------------------
	DEBUGMSG(TRUE, (_T("\tDisabling the watchdog\r\n")));
	WRITE_REGISTER_ULONG(&m_wd->REG, WD_DISABLE);
	m_enabled = FALSE;

	DEBUGMSG(TRUE, (_T("-WatchDog::Disable()\r\n")));
}


void WatchDog::Pet()
{
	DEBUGMSG(TRUE, (_T("+WatchDog::Pet()\r\n")));
	
	WRITE_REGISTER_ULONG(&m_wd->REG, WD_RESTART);

	DEBUGMSG(TRUE, (_T("-WatchDog::Pet()\r\n")));
}

BOOL WatchDog::IsEnabled()
{
	return m_enabled;
}

ULONG WatchDog::GetStatus()
{
	return (READ_REGISTER_ULONG(&m_wd->STAT) & WD_STATUS_MASK);
}

void  WatchDog::SetStatus(ULONG status)
{
	WRITE_REGISTER_ULONG(&m_wd->STAT, (status & WD_STATUS_MASK));
}


// This function maps Physical address to Virtual Address
//-------------------------------------------------------

PVOID MapPhysicalAddr(PVOID dwPhysicalAddress, DWORD dwSize)
{
	LPVOID pVirtualAddr;
	PVOID PageAlinedPhysicalAddress;
	ULONG Offset;
	DWORD PageAlignedSize;
	BOOL bCopyReturn;

	// Get Page Aligned Address
	PageAlinedPhysicalAddress = PAGE_ALIGN (dwPhysicalAddress);
	Offset = (ULONG)dwPhysicalAddress - (ULONG)PageAlinedPhysicalAddress;

	// Get page Aligned Size
	PageAlignedSize = ROUND_TO_PAGES(dwSize + Offset);

	// Create reserved block of virtual memory
	pVirtualAddr = VirtualAlloc(0, PageAlignedSize, MEM_RESERVE, PAGE_NOACCESS);

    if(pVirtualAddr == NULL) 
	{
		RETAILMSG(1,(L"	MapPhysicalAddr : Error Return. \r\n"));
        return NULL;
    }

	// Map physical memory to virtual memory
 	// then /256 and use the PAGE_PHYSICAL flag
	//----------------------------------------------
    bCopyReturn = VirtualCopy (	pVirtualAddr, 
								(LPVOID)((ULONG)PageAlinedPhysicalAddress >> 8),
								PageAlignedSize, 
								PAGE_READWRITE | PAGE_NOCACHE | PAGE_PHYSICAL);

    if(bCopyReturn == FALSE) 
	{
		RETAILMSG(1,(L"	MapPhysicalAddr : return false from VirtualCopy(). \r\n"));      
        VirtualFree( pVirtualAddr, 0, MEM_RELEASE  );
        return NULL;
    }

	pVirtualAddr = (PVOID)((PUCHAR)pVirtualAddr + Offset);

	return pVirtualAddr;

}
