// WatchDog.h
// ADS 720020-1195x
//
// Header file for the WatchDog class to use the WatchDog Timer on the EP9315 CPU
//
// jn 15 Aug 2006

#include <windows.h>

#define EP9315_OST_PHYSICAL_BASE          0x80940000  // Watchdog Timer

//WatchDog Timer
//---------------------
struct WD
{
    ULONG  REG;          // [0x00] Watch Dog Control Register
    ULONG  STAT;         // [0x04] Watch Dog Status Register
};

class WatchDog
{
	private:

		WD		*m_wd;
		ULONG	m_interval;
		BOOL	m_enabled;

	public:
		// WatchDog Constructor
		// NOTE: interval is UNUSED parameter
		WatchDog(ULONG interval);  
		
		~WatchDog();

		// Enables the WatchDog
		void Enable();

		// Disables the WatchDog
		void Disable();
		
		// Used to pet the WatchDog and reset the timer
		void Pet();

		// Tests to see if the current WatchDog is enabled
		BOOL IsEnabled();

		// Returns the 7 status storage bits of the WatchDog Status Register
		// The system can be reset by a 3 key reset, user reset, or a watchdog 
		// reset without losing the contents of this register
		ULONG GetStatus();

		// Sets the 7 status storage bits of the WatchDog Status Register
		// The system can be reset by a 3 key reset, user reset, or a watchdog 
		// reset without losing the contents of this register
		void  SetStatus(ULONG status);
};


// Watch Dog Control Register Bits
#define WD_WD		(1 << 0)  // Watch Dog Reset Status
#define WD_3KRST	(1 << 1)  // 3 Key Reset Status
#define WD_URST     (1 << 2)  // User Reset Status
#define WD_HWDIS    (1 << 3)  // Hardware Watchdog Disable
#define WD_SWDIS    (1 << 4)  // Software Watchdog Disable
#define WD_OVRID    (1 << 5)  // Software Override of the Hardware Watchdog Disable
#define WD_PLSDSN	(1 << 6)  // Pulse Disable Not

// Watch Dog Control HWORDs
#define WD_RESTART	0x5555    // Reset the watchdog, pet the watchdog
#define WD_ENABLE   0xAAAA    // Enable the watchdog
#define WD_DISABLE  0xAA55	  // Disable the watchdog

// Watch Dog Status Bits Register Masks
#define WD_STATUS_MASK    0x00007F

ULONG __inline READ_REGISTER_ULONG(PULONG Register)
{
    return *(volatile unsigned long * const)Register;
}

VOID __inline WRITE_REGISTER_ULONG(PULONG Register, ULONG value)
{
    *(volatile unsigned long * const)Register = value;
}
